@@ -19,7 +19,7 @@ package Astro::Catalog::Star;
 #    Alasdair Allan (aa@astro.ex.ac.uk)
 
 #  Revision:
-#     $Id: Star.pm,v 1.17 2003/08/26 19:53:02 aa Exp $
+#     $Id: Star.pm,v 1.22 2005/02/15 23:28:34 cavanagh Exp $
 
 #  Copyright:
 #     Copyright (C) 2002 University of Exeter. All Rights Reserved.
@@ -34,24 +34,27 @@ Astro::Catalog::Star - A generic star object in a stellar catalogue.
 
 =head1 SYNOPSIS
 
-  $star = new Astro::Catalog::Star( ID         => $id, 
-                                    Coords     => new Astro::Coords(),
-                                    Magnitudes => \%magnitudes,
-                                    MagErr     => \%mag_errors,
-                                    Colours    => \%colours,
-                                    ColErr     => \%colour_errors,
-                                    Quality    => $quality_flag,
-                                    Field      => $field,
-                                    GSC        => $in_gsc,
-                                    Distance   => $distance_to_centre,
-                                    PosAngle   => $position_angle,
-                                    X          => $x_pixel_coord,
-                                    Y          => $y_pixel_coord,
-                                    Comment    => $comment_string
-				    SpecType   => $spectral_type,
-				    StarType   => $star_type,
-				    LongStarType=> $long_star_type,
-				    MoreInfo   => $url,
+  $star = new Astro::Catalog::Star( ID           => $id, 
+                                    Coords       => new Astro::Coords(),
+                                    Magnitudes   => \%magnitudes,
+                                    MagErr       => \%mag_errors,
+                                    Colours      => \%colours,
+                                    ColErr       => \%colour_errors,
+                                    Morphology   => new Astro::Catalog::Star::Morphology(),
+                                    Quality      => $quality_flag,
+                                    Field        => $field,
+                                    GSC          => $in_gsc,
+                                    Distance     => $distance_to_centre,
+                                    PosAngle     => $position_angle,
+                                    X            => $x_pixel_coord,
+                                    Y            => $y_pixel_coord,
+                                    WCS          => new Starlink::AST(),
+                                    Comment      => $comment_string
+				    SpecType     => $spectral_type,
+				    StarType     => $star_type,
+				    LongStarType => $long_star_type,
+				    MoreInfo     => $url,
+                                    InsertDate   => new Time::Piece(),
 				  );
 
 =head1 DESCRIPTION
@@ -74,6 +77,7 @@ use warnings;
 use vars qw/ $VERSION /;
 use Carp;
 use Astro::Coords;
+use Astro::Catalog::Star::Morphology;
 
 # Register an Astro::Catalog::Star warning category
 use warnings::register;
@@ -84,7 +88,7 @@ use warnings::register;
 # This is not meant to part of the documented public interface.
 use constant DR2AS => 2.0626480624709635515647335733077861319665970087963e5;
 
-'$Revision: 1.17 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
+'$Revision: 1.22 $ ' =~ /.*:\s(.*)\s\$/ && ($VERSION = $1);
 
 # Internal lookup table for Simbad star types
 my %STAR_TYPE_LOOKUP = (
@@ -232,7 +236,7 @@ my %STAR_TYPE_LOOKUP = (
 
 =head1 REVISION
 
-$Id: Star.pm,v 1.17 2003/08/26 19:53:02 aa Exp $
+$Id: Star.pm,v 1.22 2005/02/15 23:28:34 cavanagh Exp $
 
 =head1 METHODS
 
@@ -245,24 +249,26 @@ $Id: Star.pm,v 1.17 2003/08/26 19:53:02 aa Exp $
 Create a new instance from a hash of options
 
 
-  $star = new Astro::Catalog::Star( ID         => $id, 
-                                    Coords     => new Astro::Coords(),
-                                    Magnitudes => \%magnitudes,
-                                    MagErr     => \%mag_errors,
-                                    Colours    => \%colours,
-                                    ColErr     => \%colour_errors,
-                                    Quality    => $quality_flag,
-                                    Field      => $field,
-                                    GSC        => $in_gsc,
-                                    Distance   => $distance_to_centre,
-                                    PosAngle   => $position_angle,
-                                    X          => $x_pixel_coord,
-                                    Y          => $y_pixel_coord,
-                                    Comment    => $comment_string
-				    SpecType   => $spectral_type,
-				    StarType   => $star_type,
-				    LongStarType=> $long_star_type,
-				    MoreInfo   => $url,
+  $star = new Astro::Catalog::Star( ID           => $id, 
+                                    Coords       => new Astro::Coords(),
+                                    Magnitudes   => \%magnitudes,
+                                    MagErr       => \%mag_errors,
+                                    Colours      => \%colours,
+                                    ColErr       => \%colour_errors,
+                                    Morphology   => new Astro::Catalog::Star::Morphology(),
+                                    Quality      => $quality_flag,
+                                    Field        => $field,
+                                    GSC          => $in_gsc,
+                                    Distance     => $distance_to_centre,
+                                    PosAngle     => $position_angle,
+                                    X            => $x_pixel_coord,
+                                    Y            => $y_pixel_coord,
+                                    Comment      => $comment_string
+				    SpecType     => $spectral_type,
+				    StarType     => $star_type,
+				    LongStarType => $long_star_type,
+				    MoreInfo     => $url,
+                                    InsertDate   => new Time::Piece(),
 				  );
 
 returns a reference to an Astro::Catalog::Star object.
@@ -282,6 +288,7 @@ sub new {
                       MAGERR     => {},
                       COLOURS    => {},
                       COLERR     => {},
+                      MORPHOLOGY => undef,
                       QUALITY    => undef,
                       FIELD      => undef,
                       GSC        => undef,
@@ -290,11 +297,13 @@ sub new {
 		      COORDS     => undef,
                       X          => undef,
                       Y          => undef,
+                      WCS        => undef,
                       COMMENT    => undef,
 		      SPECTYPE   => undef,
 		      STARTYPE   => undef,
 		      LONGTYPE   => undef,
 		      MOREINFO   => undef,
+                      INSERTDATE => undef,
 		    }, $class;
 
   # If we have arguments configure the object
@@ -445,14 +454,24 @@ sub ra {
   my $outc = $self->coords;
   return unless defined $outc;
 
-  # Astro::Coords inserts colons by default
-  my $outra = $outc->ra(format => 's');
+  # Astro::Coords inserts colons by default. Grab the old delimiter
+  # and number of decimal places if we're using a recent enough
+  # version of Astro::Coords.
+  my $ra = $outc->ra;
+  if (UNIVERSAL::isa( $ra, "Astro::Coords::Angle" ) ) {
 
-  # Tidy for backwards compatibility
-  $outra =~ s/:/ /g;
-  $outra =~ s/^\s*//;
+    $ra->str_delim( ' ' );
+    $ra->str_ndp( 2 );
+    return "$ra";
 
-  return $outra;
+  } else {
+
+    my $outra = $outc->ra(format => 's');
+    $outra =~ s/:/ /g;
+    $outra =~ s/^\s*//;
+
+    return $outra;
+  }
 }
 
 =item B<dec>
@@ -519,16 +538,30 @@ sub dec {
   my $outc = $self->coords;
   return unless defined $outc;
 
-  # Astro::Coords inserts colons by default
-  my $outdec = $outc->dec(format => 's');
-  $outdec =~ s/:/ /g;
-  $outdec =~ s/^\s*//;
+  # Astro::Coords inserts colons by default. Grab the old delimiter
+  # and number of decimal places if we're using a recent enough
+  # version of Astro::Coords.
+  my $dec = $outc->dec;
+  if( UNIVERSAL::isa( $dec, "Astro::Catalog::Angle" ) ) {
 
-  # require leading sign for backwards compatibility
-  # Sign will be there for negative
-  $outdec = (substr($outdec,0,1) eq '-' ? '' : '+' ) . $outdec;
+    $dec->str_delim( ' ' );
+    $dec->str_ndp( 2 );
+    $dec = "$dec";
+    $dec = (substr($dec,0,1) eq '-' ? '' : '+' ) . $dec;
+    return $dec;
 
-  return $outdec;
+  } else {
+
+    my $outdec = $outc->dec(format => 's');
+    $outdec =~ s/:/ /g;
+    $outdec =~ s/^\s*//;
+
+    # require leading sign for backwards compatibility
+    # Sign will be there for negative
+    $outdec = (substr($outdec,0,1) eq '-' ? '' : '+' ) . $outdec;
+
+    return $outdec;
+  }
 }
 
 =item B<magnitudes>
@@ -779,6 +812,34 @@ sub get_colourerr {
   return $col_error;
 }
 
+=item B<morphology>
+
+Get or set the morphology of the star as an C<Astro::Catalog::Star::Morphology>
+object.
+
+  $star->morphology( $morphology );
+
+The object returned by this method is the actual object stored
+inside this Star object and not a clone. If the morphology
+is changed through this object the morphology of the star is
+also changed.
+
+=cut
+
+sub morphology {
+  my $self = shift;
+  if (@_) {
+    my $m = shift;
+    croak "Morphology must be an Astro::Catalog::Star::Morphology object"
+      unless UNIVERSAL::isa($m, "Astro::Catalog::Star::Morphology");
+
+    # Store the new coordinate object
+    # Storing it late stops looping from the id and comment methods
+    $self->{MORPHOLOGY} = $m;
+  }
+  return $self->{MORPHOLOGY};
+}
+
 =item B<quality>
 
 Return (or set) the quality flag of the star
@@ -821,6 +882,13 @@ sub quality {
     
     # Anyway...
     my $quality = shift;
+    
+    # Shouldn't happen?
+    unless ( defined $quality ) {
+      $self->{QUALITY} = undef;
+      return undef;
+    }  
+
     if ( $quality =~ /^[A-Z][A-Z][A-Z]$/ ) {
        
        $_ = $quality;
@@ -939,6 +1007,31 @@ sub x {
   if (@_) {
     $self->{X} = shift;
   }
+
+  if( ! defined( $self->{X} ) &&
+      defined( $self->wcs ) &&
+      defined( $self->coords ) ) {
+
+    # We need to get a template FK5 SkyFrame to be able to convert
+    # properly between RA/Dec and X/Y, but we can only do this if
+    # we load Starlink::AST. So that we don't have a major dependency
+    # on that module, load it here at runtime.
+    eval( "require Starlink::AST" );
+    if( $@ ) {
+      croak "Attempted to convert from RA/Dec to X position and cannot load Starlink::AST. Error: $@";
+    }
+    my $template = new Starlink::AST::SkyFrame( "System=FK5" );
+    my $wcs = $self->wcs;
+    my $frameset = $wcs->FindFrame( $template, "" );
+    if( ! defined( $frameset ) ) {
+      croak "Could not find FK5 SkyFrame to do RA/Dec to X position translation";
+    }
+    my( $ra, $dec ) = $self->coords->radec();
+    my( $x, $y ) = $frameset->Tran2( [$ra->radians],
+                                     [$dec->radians],
+                                     0 );
+    $self->{X} = $x->[0];
+  }
   return $self->{X};
 }
 
@@ -956,9 +1049,56 @@ sub y {
   if (@_) {
     $self->{Y} = shift;
   }
+
+  if( ! defined( $self->{Y} ) &&
+      defined( $self->wcs ) &&
+      defined( $self->coords ) ) {
+
+    # We need to get a template FK5 SkyFrame to be able to convert
+    # properly between RA/Dec and X/Y, but we can only do this if
+    # we load Starlink::AST. So that we don't have a major dependency
+    # on that module, load it here at runtime.
+    eval( "require Starlink::AST" );
+    if( $@ ) {
+      croak "Attempted to convert from RA/Dec to Y position and cannot load Starlink::AST. Error: $@";
+    }
+    my $template = new Starlink::AST::SkyFrame( "System=FK5" );
+    my $wcs = $self->wcs;
+    my $frameset = $wcs->FindFrame( $template, "" );
+    if( ! defined( $frameset ) ) {
+      croak "Could not find FK5 SkyFrame to do RA/Dec to Y position translation";
+    }
+    my( $ra, $dec ) = $self->coords->radec();
+    my( $x, $y ) = $frameset->Tran2( [$ra->radians],
+                                     [$dec->radians],
+                                     0 );
+    $self->{Y} = $y->[0];
+  }
+
   return $self->{Y};
 }
 
+=item B<wcs>
+
+Return (or set) the WCS associated with the star.
+
+  $wcs = $star->wcs;
+  $star->wcs( $wcs );
+
+The WCS is a C<Starlink::AST> object.
+
+=cut
+
+sub wcs {
+  my $self = shift;
+  if( @_ ) {
+    my $wcs = shift;
+    if( UNIVERSAL::isa( $wcs, "Starlink::AST" ) ) {
+      $self->{WCS} = $wcs;
+    }
+  }
+  return $self->{WCS};
+}
 
 =item B<comment>
 
@@ -1062,6 +1202,24 @@ sub moreinfo {
   return $self->{MOREINFO};
 }
 
+=item B<insertdate>
+
+The time the information for the star in question was gathered. This
+is different from the time of observation of the star.
+
+  $insertdate = $star->insertdate;
+
+This is a C<Time::Piece> object.
+
+=cut
+
+sub insertdate {
+  my $self = shift;
+  if( @_ ) {
+    $self->{INSERTDATE} = shift;
+  }
+  return $self->{INSERTDATE};
+}
 
 # C O N F I G U R E -------------------------------------------------------
 
